@@ -38,6 +38,7 @@
 #include "eog-preferences.h"
 #include "eog-window.h"
 #include "util.h"
+#include "zoom.h"
 #include "Eog.h"
 
 /* Default size for windows */
@@ -64,6 +65,7 @@ struct _EogWindowPrivate {
 
 	/* vbox */
 	GtkWidget           *box;
+	GtkWidget           *ctrl_widget;
 
 	/* preference dialog, if available */
 	GtkWidget           *pref_dlg;
@@ -94,7 +96,7 @@ static void open_dnd_files (EogWindow *window, gboolean need_new_window);
 static BonoboWindowClass *parent_class;
 
 /* The list of all open windows */
-static GList *window_list;
+static GList *window_list = NULL;
 
 /* Drag target types */
 enum {
@@ -201,18 +203,7 @@ verb_FileCloseWindow_cb (BonoboUIComponent *uic, gpointer user_data, const char
 static void
 verb_FileExit_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
-	GList *l;
-	EogWindow *w;
-
-	/* Destroy windows and exit */
-	for (l = window_list; l != NULL; l = l->next) {
-		w = EOG_WINDOW (l->data);
-		gtk_widget_destroy (GTK_WIDGET (w));
-	}
-	g_list_free (window_list);
-	window_list = NULL;
-
-	bonobo_main_quit ();
+	eog_window_close_all ();
 }
 
 static void
@@ -328,21 +319,13 @@ static void
 activate_uri_cb (BonoboControlFrame *control_frame, const char *uri, gboolean relative, gpointer data)
 {
 	EogWindow *window;
-	char *path;
 
 	g_return_if_fail (uri != NULL);
 
 	window = EOG_WINDOW (eog_window_new ());
 	
-	if (g_ascii_strncasecmp ("file:", uri, 5) == 0) 		
-		path = g_strdup ((uri+5));
-	else
-		path = g_strdup (uri);
-
-	eog_window_open (window, path);
+	eog_window_open (window, uri);
 	gtk_widget_show (GTK_WIDGET (window));
-
-	g_free (path);
 }
 
 GType
@@ -382,12 +365,11 @@ eog_window_destroy (GtkObject *object)
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (EOG_IS_WINDOW (object));
 
-	if (getenv ("DEBUG_EOG"))
-		g_message ("Destroying EogWindow...");
-
 	window = EOG_WINDOW (object);
 	priv = window->priv;
 
+	window_list = g_list_remove (window_list, window);
+
 	if (priv->uri) {
 		g_free (priv->uri);
 		priv->uri = NULL;
@@ -496,6 +478,7 @@ eog_window_init (EogWindow *window)
 		NULL);
 
 	window_list = g_list_prepend (window_list, window);
+	priv->ctrl_widget = NULL;
 
 	g_object_set (G_OBJECT (window), "allow_shrink", TRUE, NULL);
 }
@@ -539,13 +522,9 @@ eog_window_key_press (GtkWidget *widget, GdkEventKey *event)
 static gboolean
 eog_window_has_contents (EogWindow *window)
 {
-	EogWindowPrivate *priv;
-
 	g_return_val_if_fail (EOG_IS_WINDOW (window), FALSE);
 
-	priv = window->priv;
-
-	return (bonobo_control_frame_get_widget (priv->ctrl_frame) != NULL);
+	return (bonobo_control_frame_get_control (window->priv->ctrl_frame) != CORBA_OBJECT_NIL);
 }
 
 /* Drag_data_received handler for windows */
@@ -689,7 +668,6 @@ eog_window_construct (EogWindow *window)
 {
 	EogWindowPrivate *priv;
 	BonoboUIContainer *ui_container;
-	GtkWidget *widget;
 
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (EOG_IS_WINDOW (window));
@@ -726,13 +704,6 @@ eog_window_construct (EogWindow *window)
 	g_signal_connect (G_OBJECT (priv->ctrl_frame), "activate_uri",
 			  (GtkSignalFunc) activate_uri_cb, NULL);
 
-	widget = bonobo_control_frame_get_widget (priv->ctrl_frame);
-	if (!widget)
-		g_assert_not_reached ();
-
-	gtk_box_pack_start (GTK_BOX (priv->box), widget, TRUE, TRUE, 0);
-	gtk_widget_show (widget);
-
 	set_drag_dest (window);
 
 	/* set default geometry */
@@ -755,8 +726,6 @@ eog_window_close (EogWindow *window)
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (EOG_IS_WINDOW (window));
 
-	window_list = g_list_remove (window_list, window);
-
 	gtk_widget_destroy (GTK_WIDGET (window));
 
 	if (!window_list)
@@ -1072,6 +1041,96 @@ get_collection_control_list (GList *text_uri_list)
 	return control;
 }
 
+void
+adapt_shell_size_to_control (EogWindow *window, Bonobo_Control control)
+{
+	CORBA_Environment ev;
+	EogWindowPrivate *priv;
+	Bonobo_PropertyBag pb;
+	gint32 width, height;
+	gint32 image_width, image_height;
+	int sw, sh;
+	Bonobo_Zoomable zi;
+	double zoom_x;
+	double zoom_y;
+	gboolean need_zoom;
+	int req_width, req_height;
+	int xthick, ythick;
+
+	g_return_if_fail (EOG_IS_WINDOW (window));
+
+	CORBA_exception_init (&ev);
+	priv = window->priv;
+
+	/* calculate initial eog window size */
+	pb = bonobo_control_frame_get_control_property_bag (priv->ctrl_frame, &ev);
+	if (pb == CORBA_OBJECT_NIL) return;
+
+		
+	/* FIXME: We try to obtain the desired size of the component
+	 * here. The image/width image/height properties aren't
+	 * generally available in controls and work only with the
+	 * eog-image-viewer component!  
+	 */
+	image_width = bonobo_pbclient_get_long (pb, "image/width", &ev);
+	image_height = bonobo_pbclient_get_long (pb, "image/height", &ev);
+	
+	sw = gdk_screen_width ();
+	sh = gdk_screen_height ();
+	need_zoom = FALSE;
+
+	if (image_width >= sw) {
+		width = 0.75 * sw;
+		need_zoom = TRUE;
+	}
+	else {
+		width = image_width;
+	}
+	if (image_height >= sh) {
+		height = 0.75 * sh;
+		need_zoom = TRUE;
+	}
+	else {
+		height = image_height;
+	}
+
+	if (!width  || !height) {
+		bonobo_object_release_unref (pb, &ev);
+		return;
+	}		
+
+	/* this is the size of the frame around the vbox */
+	xthick = priv->box->style->xthickness;
+	ythick = priv->box->style->ythickness;
+	
+	req_height = 
+		height + 
+		(GTK_WIDGET(window)->allocation.height - priv->box->allocation.height) +
+		priv->statusbar->allocation.height + 
+		2 * ythick;
+	
+	req_width = 
+		width + 
+		(GTK_WIDGET(window)->allocation.width - priv->box->allocation.width) +
+		2 * xthick;
+
+	gtk_window_resize (GTK_WINDOW (window), req_width, req_height);
+
+	if (need_zoom) {
+		zi = Bonobo_Unknown_queryInterface (control, "IDL:Bonobo/Zoomable:1.0", &ev);
+		if (zi != CORBA_OBJECT_NIL) {
+			double zoom_level;
+			zoom_level = zoom_fit_scale (width, height, 
+						     image_width, image_height, TRUE);
+			Bonobo_Zoomable_setLevel (zi, zoom_level, &ev);
+			bonobo_object_release_unref (zi, &ev);
+		}
+	}
+	
+	bonobo_object_release_unref (pb, &ev);
+	CORBA_exception_free (&ev);
+}
+
 static void 
 add_control_to_ui (EogWindow *window, Bonobo_Control control)
 {
@@ -1085,8 +1144,22 @@ add_control_to_ui (EogWindow *window, Bonobo_Control control)
 	priv = window->priv;
 	CORBA_exception_init (&ev);
 
+	/* bind and view new control widget */
 	bonobo_control_frame_bind_to_control (priv->ctrl_frame, control, &ev);
 	bonobo_control_frame_control_activate (priv->ctrl_frame);
+	if (control != CORBA_OBJECT_NIL && priv->ctrl_widget == NULL) {
+		priv->ctrl_widget = bonobo_control_frame_get_widget (priv->ctrl_frame);
+		if (!priv->ctrl_widget)
+			g_assert_not_reached ();
+
+		gtk_box_pack_start (GTK_BOX (priv->box), priv->ctrl_widget, TRUE, TRUE, 0);
+		gtk_widget_show (priv->ctrl_widget);
+	}
+	else if (control == CORBA_OBJECT_NIL && priv->ctrl_widget != NULL) {
+		gtk_container_remove (GTK_CONTAINER (priv->box), 
+				      GTK_WIDGET (priv->ctrl_widget));
+		priv->ctrl_widget = NULL;
+	}
 
 	/* update sensitivity of the properties menu item */
 	prop_control = Bonobo_Unknown_queryInterface (control, 
@@ -1105,6 +1178,8 @@ add_control_to_ui (EogWindow *window, Bonobo_Control control)
 	 */
 	bonobo_ui_component_set_prop (priv->ui_comp, "/menu/View", "hidden", "0", &ev);
 
+	adapt_shell_size_to_control (window, control);
+
 	CORBA_exception_free (&ev);
 
 	/* retrieve control properties and install listeners */
@@ -1272,3 +1347,24 @@ eog_window_get_uri (EogWindow *eog_window)
 	priv = eog_window->priv;
 	return priv->uri;
 }
+
+/**
+ * eog_window_close_all:
+ * 
+ * Closes all EOG windows, causing the program to exit.
+ **/
+void
+eog_window_close_all (void)
+{
+	while (1) {
+		GList *l;
+		EogWindow *window;
+
+		l = window_list;
+		if (!l)
+			break;
+
+		window = EOG_WINDOW (l->data);
+		eog_window_close (window);
+	}
+}
